#ifndef BINDING_TABLE_H
#define BINDING_TABLE_H

/*
 * \author    dresden elektronik ingenieurtechnik gmbh: http://www.dresden-elektronik.de
 * \author    Support email: wireless@dresden-elektronik.de
 *
 * Copyright (c) 2016-2020, dresden elektronik ingenieurtechnik gmbh. All rights reserved.
 *
 * Licensed under dresden elektronik's Limited License Agreement --> deEULA.txt
 */

#include "deconz/types.h"
#include "deconz/aps.h"

namespace deCONZ {

class DECONZ_DLLSPEC Binding
{
public:
    Binding() : m_srcEndpoint(0xFF), m_dstEndpoint(0xFF), m_cluster(0xFFFF), m_dstAddrMode(deCONZ::ApsExtAddress) {}
    deCONZ::Address &srcAddress() { return m_srcAddr; }
    const deCONZ::Address &srcAddress() const { return m_srcAddr; }
    uint8_t srcEndpoint() const { return m_srcEndpoint; }
    void setSrcEndpoint(uint8_t ep) { m_srcEndpoint = ep; }
    uint16_t cluster() const { return m_cluster; }
    void setCluster(uint16_t cluster) { m_cluster = cluster; }
    deCONZ::ApsAddressMode dstAddressMode() const { return m_dstAddrMode; }
    void setDstAddressMode(deCONZ::ApsAddressMode mode) { m_dstAddrMode = mode; }
    deCONZ::Address &dstAddress() { return m_dstAddr; }
    const deCONZ::Address &dstAddress() const { return m_dstAddr; }
    uint8_t dstEndpoint() const { return m_dstEndpoint; }
    void setDstEndpoint(uint8_t ep) { m_dstEndpoint = ep; }
    Binding reverse() const
    {
        Binding rev;

        rev.srcAddress() = dstAddress();
        rev.dstAddress() = srcAddress();
        rev.setSrcEndpoint(dstEndpoint());
        rev.setDstEndpoint(srcEndpoint());
        rev.setDstAddressMode(dstAddressMode());
        rev.setCluster(cluster());

        return rev;
    }

    bool operator==(const Binding &other) const
    {
        if (dstAddress()  == other.dstAddress() &&
            dstAddressMode() == other.dstAddressMode() &&
            srcAddress()  == other.srcAddress() &&
            srcEndpoint() == other.srcEndpoint() &&
            dstEndpoint() == other.dstEndpoint() &&
            cluster()     == other.cluster())
        {
            return true;
        }

        return false;
    }

    bool operator !=(const Binding &other)
    {
        return !(*this == other);
    }

private:
    uint8_t m_srcEndpoint;
    uint8_t m_dstEndpoint;
    uint16_t m_cluster;
    deCONZ::ApsAddressMode m_dstAddrMode;
    deCONZ::Address m_srcAddr;
    deCONZ::Address m_dstAddr;
};

class DECONZ_DLLSPEC BindingTable
{
public:
    BindingTable();
    void addBinding(const Binding &binding);
    void removeBinding(const Binding &binding);
    void merge(const BindingTable &other);
    int entryCount() const { return m_entryCount; }
    void setEntryCount(int count) { m_entryCount = count; }
    int startIndex() const { return m_startIndex; }
    void setStartIndex(int index) { m_startIndex = index; }
    int size() const { return m_table.size(); }
    Binding &binding(int i) { return m_table[i]; }
    const Binding &binding(int i) const { return m_table[i]; }
    bool contains(const Binding &binding);

private:
    int m_entryCount;
    int m_startIndex;
    QList<Binding> m_table;
};

} // namespace deCONZ

Q_DECLARE_METATYPE(deCONZ::BindingTable)


#endif // BINDING_TABLE_H

